* mssm_ini.F
* all parameters for the MSSM are initialized here
* this file is part of FormCalc
* Christian Schappacher, 17 Feb 99
* last changed 7 Jul 00 th


* the following switches are usually set in process.h
*#define SQUARKMIXING
*#define COMPLEXPARAMETERS
*#define SM_ONLY
*#define NO_EXCLUSION_LIMITS

#define MSQ MSusy
#define MSU MSusy
#define MSD MSusy
#define MSE MSusy
#define MSL MSusy

#define MSQ2 MSQ**2
#define MSU2 MSU**2
#define MSD2 MSD**2
#define MSE2 MSE**2
#define MSL2 MSL**2


* Exclusion limits (only checked if NO_EXCLUSION_LIMITS is not set):

* hep-ex/9910007 (L3) and hep-ex/9909051 (OPAL)
#define MNeutralino_LOWERBOUND 30

* hep-ex/9910007 (L3) and hep-ex/9909051 (OPAL)
#define MChargino_LOWERBOUND 90

* hep-ex/9910006 (L3)
#define MSlepton_LOWERBOUND 70

* hep-ex/9910020 (L3)
#define MSquark_LOWERBOUND 150

* hep-ex/9910020 (L3)
#define MSbottom_LOWERBOUND 70

* hep-ex/9910020 (L3)
#define MStop_LOWERBOUND 80

* hep-ex/9908016 (ALEPH) and ???
#define Mh0_LOWERBOUND 85

* hep-ex/9910020 (L3) and PDG
#define MGl_LOWERBOUND 175

* NB: the delta-rho limit is really .0013. However, no 2-loop terms
* have been included here (yet), and this is compensated by a
* somewhat smaller limit (KA-TP-29-1996).

#define DeltaRho_UPPERBOUND .0012D0


* tolerance for checking whether diagonalization of neutralino mass
* matrices went ok:

#define TOLERANCE 1D-10


#define SW4 (SW2**2)

#define model_ini sm_ini
#include "sm_ini.F"
#undef model_ini

************************************************************************

	subroutine model_ini(Ecms, reset, *)
	implicit none
	double precision Ecms
	logical reset

#include "model.h"

	integer i, j
	double precision MTr, MTr2
	double precision C2B2
	double precision A, B, Alfa_Ecms, Alfas_MT
	double precision lambda, M_S, M_SR, m11, m12, m22
	double precision pb, pf, pg, pgp, p1h, p2h, p2hp
	double precision mts, m11botmix, m12botmix, m22botmix
	double precision m11rest, m12rest, m22rest, pref, thetafunc
	double precision ttt, vvv, xttilde, delmh, w_t, s_t, l_t
	double precision DeltaRho
	double complex MULR, MDLR

	integer nc, ng
	parameter (nc = 3, ng = 3)

	double precision ALPHAS2
	external ALPHAS2

	double precision f0
	double precision x, y

	f0(x, y) = x + y - (2*x*y)/(x - y)*Log(x/y)


	call sm_ini(Ecms, reset, *999)

*-----------------------------------------------------------------------
* MSSM input:

	M_2 = 500

	Alfa_Ecms = Alfa/(1D0 - (0.059461D0 +
     +    40*Alfa/(9D0*pi)*log(Ecms/MZ)))

* Gluino mass: GUT relation
	MGl = Alfas/Alfa_Ecms*SW2*M_2
#ifndef NO_EXCLUSION_LIMITS
	if(MGl .lt. MGl_LOWERBOUND) then
	  print *, "mssm_ini: MGl = ", MGl,
     +      " < MGl_LOWERBOUND = ", MGl_LOWERBOUND
	  return 1
	endif
#endif
	MGl2 = MGl**2

	if(.not. reset) return

	MUE = -100
c	MUE = dcmplx(0d0,100D0)
	MSusy = 200
	MA0 = 200
	MA02 = MA0**2
	TB = 5

* ATTENTION: GUT relation (Higgs Hunter's Guide p. 287)
	M_1 = 5/3D0*SW2/CW2*M_2

	do i = 1, 4
	  do j = 1, 3
#ifdef SQUARKMIXING
	    Af(i, j) = MSusy
c	    Af(i, j) = dcmplx(0D0,200D0)
#else
	    if(mod(i, 2) .eq. 1) then
	      Af(i, j) = dconjg(MUE)/TB
	    else
	      Af(i, j) = dconjg(MUE)*TB
	    endif
#endif
	  enddo
	enddo

*-----------------------------------------------------------------------
* Angles: (PART 1)

	B    = atan(TB)
	SB   = sin(B)
	CB   = cos(B)
	S2B  = sin(2*B)
	C2B  = cos(2*B)
	SB2  = SB**2
	CB2  = CB**2
	TB2  = TB**2
	C2B2 = C2B**2

	call diag(*999)

*-----------------------------------------------------------------------
* Higgs masses:
* Mh0, MHH, (MHp) are tree level masses if w_t = 0, l_t = 0,
* and s_t = 0, otherwise they are radiatively corrected!

* ATTENTION: Running top quark mass: MTr
	Alfas_MT = ALPHAS2(MT)
	MTr  = MT/(1 + 4/3D0/pi*Alfas_MT)
	MTr2 = MTr**2


#ifdef COMPLEXPARAMETERS

* Simple approximation: (for complex and/or real case)
* ~~~~~~~~~~~~~~~~~~~~~
	w_t = 3*Gmue*MTr2**2/(Sqrt(2D0)*pi**2*SB2)*
     &      .5D0*Log(1 + MSQ2*MSU2/MTr2**2 + (MSQ2 + MSU2)/MTr2)
	s_t = 0
	l_t = 0
	delmh = 0

#else

* Heinemeyer/Weiglein approximation: hep-ph/9903404 (only for real case)
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	lambda = 1/8D0 - SW2/3D0 + 4/9D0*SW4
	MULR = Af(3,3) - MUE/TB
	MDLR = Af(4,3) - MUE*TB
	M_S  = (MSQ2*MSU2 + MT2*(MSQ2 + MSU2) +
     &    MT2**2)**(1/4D0)
	M_SR = (MSQ2*MSU2 + MTr2*(MSQ2 + MSU2) +
     &    MTr2**2)**(1/4D0)

	mts = MT2/M_S**2

	m11 = -Gmue*Sqrt(2D0)*MZ2**2*lambda*CB2/pi**2 * Log(mts)

	m12 = Gmue*Sqrt(2D0)*MZ2/TB/pi**2 *
     &    (-3*MT2/8D0 + MZ2*lambda*SB2) * Log(mts)

	m22 = -Gmue*Sqrt(2D0)*MT2**2/(8D0*pi**2*SB2) *
     &    ( -2*MZ2/MT2 + 11/10D0*(MZ2/MT2)**2
     &       + (12 - 6*SB2*MZ2/MT2
     &       + 8*lambda*SB2**2*(MZ2/MT2)**2) * Log(mts)
     &       + MULR**2/M_S**2 * (-12 + 4*MZ2/MT2 + 6*mts)
     &       + MULR**4/M_S**4 * (1 - 4*mts + 3*mts**2)
     &       + MULR**6/M_S**6 * (3/5D0*mts
     &           - 12/5D0*mts**2 + 2*mts**3)
     &       + MULR**8/M_S**8 * (3/7D0*mts**2
     &           - 12/7D0*mts**3 + 3/2D0*mts**4) )
* 2-loop contribution:
     &    - Gmue*Sqrt(2D0)*Alfas_MT*MTr2**2/(pi**3*SB2)
     &      * (  3D0*(Log(MTr2/M_SR**2))**2 
     &         - 6D0*Log(MTr2/M_SR**2)
     &         - 6D0*MULR/M_SR
     &         - 3D0*MULR**2/M_SR**2*Log(MTr2/M_SR**2)
     &         + 3D0*MULR**4/4D0/M_SR**4 )

	pb = nc*(1 + 4*(-1/3D0)*SW2 + 8*(-1/3D0)**2*SW4)
	pf = nc*(ng - 1)*(2 - 4*SW2 
     $    + 8*((2/3D0)**2 + (-1/3D0)**2)*SW4)
     $    + ng*(2 - 4*SW2 + 8*SW4)
	pg   = -44 + 106*SW2 - 62*SW4
	pgp  = 10 + 34*SW2 - 26*SW4
	p1h  = -9*C2B**4 + (1 - 2*SW2 + 2*SW4)*C2B2
	p2h  = -10 + 2*SW2 - 2*SW4
	p2hp =  8 - 22*SW2 + 10*SW4

	thetafunc = 0
	if(MA0 .ge. MZ) thetafunc = 1

	pref = Gmue*MW2*MZ2/(12D0*Sqrt(2D0)*pi**2*CW2)

* All other sectors except for the sbottom mixing:
	m11rest = pref*CB2*
     $          ( (12*nc*MB2**2/(MZ2**2*CB**4) 
     $             - 6*nc*MB2/(MZ2*CB2) 
     $             + pb + pf + pg + p2h)*Log(MSQ2/MZ2)
     $           + thetafunc*(p1h - p2h)*Log(MA02/MZ2) )
	m12rest = -pref * SB * CB *
     $          ( (pb - 3*nc*MB2/(MZ2*CB2) 
     $             + pf + pgp + p2hp)*Log(MSQ2/MZ2) 
     $           - thetafunc*(p1h + p2hp)*Log(MA02/MZ2) )
	m22rest = pref*SB2 * 
     $          ( (pb + pf + pg + p2h)*Log(MSQ2/MZ2) 
     $             + thetafunc*(p1h - p2h)*Log(MA02/MZ2) )


* The bottom mixing at 1-loop:
	m11botmix = Gmue*nc/(4D0*Sqrt(2D0)*pi**2*MSQ2)
     &  * (4*MB**4*Af(4, 3)*MDLR/CB2
     &  * (1 - Af(4,3)*MDLR/12D0/MSQ2)
     &     - MZ2*MB2*Af(4, 3)*(MDLR + Af(4, 3)/3D0) )
	m12botmix = -Gmue*nc/(8D0*Sqrt(2D0)*pi**2*MSQ2)
     &  * ( 4*MB2**2*MUE*MDLR/CB2*(1 - Af(4, 3)*MDLR/6D0/MSQ2) 
     &     - MZ2*MB2*TB
     &  * (MDLR*(Af(4, 3) + MUE/TB) + (MUE**2 + Af(4, 3)**2)/3D0) )
	m22botmix = Gmue*nc/(4D0*Sqrt(2D0)*pi**2*MSQ2)
     &  * ( - MB2**2*MUE**2*MDLR**2/(3D0*CB2*MSQ2)
     &      - MZ2*MB2*MUE*TB*(MDLR + MUE*TB/3D0) )

	w_t = m22 + m22rest + m22botmix
	l_t = m12 + m12rest + m12botmix
	s_t = m11 + m11rest + m11botmix

* Including the leading Yukawa term for the light Higgs mass this
* term is (partially) taken from Carena, Quiros, Wagner,
* Nucl. Phys. B461 (1996) 407

	xttilde = (((MSf2(2,3,3) - MSf2(1,3,3))/(4D0*MTr2)
     $            * (2*USf(2,1,3,3)
     $            * Sqrt(1 - (ABS(USf(2,1,3,3)))**2))**2 )**2
     $            * (2 - (MSf2(2,3,3) 
     $                + MSf2(1,3,3))/(MSf2(2,3,3) - MSf2(1,3,3))
     $                  * Log(MSf2(2,3,3)/MSf2(1,3,3)))
     $           + (MSf2(2,3,3) - MSf2(1,3,3))/(2D0 * MTr2)
     $            * (2*USf(2,1,3,3) 
     $            * Sqrt(1 - (ABS(USf(2,1,3,3)))**2))**2
     $                 * Log(MSf2(2,3,3)/MSf2(1,3,3)) )
	ttt = 0.5D0*Log(MSf2(2,3,3)*MSf2(1,3,3)/MTr2**2)
	vvv = 174.1D0
	delmh = ( 3/(4D0*pi**2)*MTr2**2/vvv**2
     $         * (1/(16D0*pi**2)*3D0/2D0 * MTr2/vvv**2
     $         * ttt*(xttilde + ttt)) )
#endif

*-----------------------------------------------------------------------

	Mh0 = (MA02 + MZ2 + w_t + s_t)/2D0 
     &          - Sqrt(((MA02 + MZ2)**2 + (w_t - s_t)**2)/4D0
     &          - MA02*MZ2*C2B2 + (w_t - s_t)*C2B*(MA02 - MZ2)/2D0
     &           - l_t*S2B*(MA02 + MZ2) + l_t**2) + delmh
	if(Mh0 .lt. 0) then
	  print *, "mssm_ini: negative Mh0^2 detected."
	  return 1
	endif
	Mh0 = Sqrt(Mh0)

#ifndef NO_EXCLUSION_LIMITS
	if(Mh0 .lt. Mh0_LOWERBOUND) then
	  print *, "mssm_ini: Mh0 = ", Mh0,
     +      " < Mh0_LOWERBOUND = ", Mh0_LOWERBOUND
	  return 1
	endif
#endif

	MHH = Sqrt((MA02 + MZ2 + w_t + s_t)/2D0 
     &          + Sqrt(((MA02 + MZ2)**2 + (w_t - s_t)**2)/4D0
     &          - MA02*MZ2*C2B2 + (w_t - s_t)*C2B*(MA02 - MZ2)/2D0
     &          - l_t*S2B*(MA02 + MZ2) + l_t**2))

* M.A. Diaz: PhD Thesis p. 36-40.
	if(MA0 .le. 250 .and. MSQ .ge. 250) then
	  MHp = Sqrt(MA02 + MW2
     &      + 3/4D0/pi*Alfa/SW2/MW2
     &        * (2*MTr2*MB2/(SB2*CB2)
     &           - MW2*(MTr2/SB2 + MB2/CB2) + 2/3D0*MW2**2)
     &        * Log(MSQ/MTr)
     &      + 5/2D0/pi*Alfa*MW2/CW2*Log(MSQ/MW))
	else
	  MHp = Sqrt(MA02 + MW2)
	endif

	Mh02 = Mh0**2
	MHH2 = MHH**2
	MHp2 = MHp**2
	MG0 = MZ
	MGp = MW
	MG02 = MZ2
	MGp2 = MW2

*-----------------------------------------------------------------------
* Angles: (PART 2)

#ifdef SM_ONLY

* With SM_ONLY defined, the values of the MSSM parameters are set
* such that the SM is reproduced. See Higgs Hunter's Guide p. 356.
* Note one tricky thing: the Higgs mass Mh0 is first calculated as
* it would be in the MSSM (to get the same value when comparing
* MSSM <-> SM), but then beta is reverse-engineered to be consistent
* with the SM relations. In other words, we start with the MSSM-
* value of beta, calculate Mh0, and then redefine beta to fulfill
* cos(2 beta) = Mh0/MZ. In principle, beta could also be fixed such
* that cos(2 beta) = -Mh0/MZ, but this is insignificant here since
* we only need beta to be consistent with the values of Mh0 and MZ.

	C2B = Mh0/MZ
	B   = .5D0*acos(C2B)
	SB  = sin(B)
	CB  = cos(B)
	TB  = tan(B)
	S2B = sin(2*B)

	A   = 0
	SA  = -CB
	CA  = SB
	S2A = -S2B
	C2A = -C2B
	SAB = -C2B
	CAB = S2B
	SBA = 1
	CBA = 0
#else
	A = (-(MA02 + MZ2)*SB*CB + l_t)/
     &        (MZ2*CB2 + MA02*SB2 + s_t - Mh02)
	if(A .le. 0) then
	  A = atan(A)
	else
	  A = atan(A) - pi
	endif
	SA  = sin(A)
	CA  = cos(A)
	S2A = sin(2*A)
	C2A = cos(2*A)
	SAB = sin(A + B)
	CAB = cos(A + B)
	SBA = sin(B - A)
	CBA = cos(B - A)
#endif

	SA2  = SA**2
	CA2  = CA**2

*-----------------------------------------------------------------------
* Calculation of Delta Rho:

#ifndef NO_EXCLUSION_LIMITS
	DeltaRho = 3*Gmue/(8D0*Sqrt(2D0)*pi**2) *
     &      ( - (ABS(USf(1,1,3,3)))**2 * (ABS(USf(2,1,3,3)))**2 
     &                 * f0(MSf2(1,3,3), MSf2(2,3,3))
     &        - (ABS(USf(1,1,4,3)))**2 * (ABS(USf(2,1,4,3)))**2 
     &                 * f0(MSf2(1,4,3), MSf2(2,4,3))
     &        + (ABS(USf(1,1,3,3)))**2 * (ABS(USf(1,1,4,3)))**2 
     &                 * f0(MSf2(1,3,3), MSf2(1,4,3))
     &        + (ABS(USf(1,1,3,3)))**2 * (ABS(USf(2,1,4,3)))**2
     &                 * f0(MSf2(1,3,3), MSf2(2,4,3))
     &        + (ABS(USf(1,1,4,3)))**2 * (ABS(USf(2,1,3,3)))**2 
     &                 * f0(MSf2(2,3,3), MSf2(1,4,3))
     &        + (ABS(USf(2,1,3,3)))**2 * (ABS(USf(2,1,4,3)))**2
     &                 * f0(MSf2(2,3,3), MSf2(2,4,3)) )

	if(DeltaRho .gt. DeltaRho_UPPERBOUND) then
	  print *, "mssm_ini: DeltaRho = ", DeltaRho,
     +      " > DeltaRho_UPPERBOUND = ", DeltaRho_UPPERBOUND
	  return 1
	endif
#endif

*-----------------------------------------------------------------------

#ifdef DEBUG

	print *, "SUSY parameters"
	print *, "----------------------------"
	print *, "1/alpha(s) =", 1/Alfa_Ecms
	print *, "alpha_s(MT) =", Alfas_MT
	print *, "running top mass MTr =", MTr
	print *, "MSusy =", MSusy
	print *, "M_1 =", M_1
	print *, "M_2 =", M_2
	print *, "MUE =", MUE
	print *, "TB =", TB
	print *, "Af(top) =", Af(3, 3)
	print *, "Af(bottom) =", Af(4, 3)
	print *, "----------------------------"
	print *, "Higgs masses:"
#ifdef COMPLEXPARAMETERS
	print *, "(using simple approximation for Higgs mass)"
#else
	print *, "(using 2-loop approximation for Higgs mass)"
#endif
	print *, "Mh0 =", Mh0
	print *, "MHH =", MHH
	print *, "MHp =", MHp
	print *, "MG0 =", MG0
	print *, "MGp =", MGp
	print *, "MA0 =", MA0
	print *, "----------------------------"
	print *, "Sfermion masses:"
#ifdef SQUARKMIXING
	print *, "(non-diagonal sfermion mass matrix: 1-2 basis)"
#else
	print *, "(diagonal sfermion mass matrix: L-R basis)"
#endif
3	format(" MSf(1|2, ", I1, ", ", I1, ") =", 2G24.15)
	do j = 1, 4
	  do i = 1, 3
	    print 3, j, i, MSf(1, j, i), MSf(2, j, i)
	  enddo
	enddo
	print *, "----------------------------"
	print *, "Chargino masses:"
	print *, "MCha_1 =", MCha(1)
	print *, "MCha_2 =", MCha(2)
	print *, "----------------------------"
	print *, "Neutralino masses:"
	print *, "MNeu_1 =", MNeu(1)
	print *, "MNeu_2 =", MNeu(2)
	print *, "MNeu_3 =", MNeu(3)
	print *, "MNeu_4 =", MNeu(4)
	print *, "----------------------------"
	print *, "Gluino mass:"
	print *, "MGl =", MGl
	print *, "============================"

#endif
	return

999	return 1
	end

************************************************************************

	subroutine diag(*)
	implicit none

#include "model.h"

	integer lwork, i, j, k, l, iiso3, svdinfo
	parameter (lwork = 16)
	double complex X(2, 2), Y(4, 4), work(lwork), rwork(lwork)
	double complex USVD(4, 4), VHSVD(4, 4)
	double complex Ysave(4, 4), fac, unitary
	double precision yukfac(0:1), mdoub2(4), msing2(4)
	double precision mf(4, 3)
	double precision upcharge, downcharge, charge(4)
	parameter (upcharge = 2/3D0, downcharge = -1/3D0)
	data charge / 0D0, -1D0, upcharge, downcharge /

	mf(2, 1) = ME
	mf(3, 1) = MU
	mf(4, 1) = MD
	mf(2, 2) = MM
	mf(3, 2) = MC
	mf(4, 2) = MS
	mf(2, 3) = ML
	mf(3, 3) = MT
	mf(4, 3) = MB

*-----------------------------------------------------------------------
* Sfermion masses:

	yukfac(0) = TB
	yukfac(1) = 1/TB
	mdoub2(2) = MSL2
	mdoub2(3) = MSQ2
	mdoub2(4) = MSQ2
	msing2(2) = MSE2
	msing2(3) = MSU2
	msing2(4) = MSD2

	do i = 2, 4
	  iiso3 = mod(i, 2)
	  do j = 1, 3
	    USf(1, 1, i, j) = mdoub2(i) +
     +        MZ2*C2B*(iiso3 - .5D0 - charge(i)*SW2) + mf(i, j)**2
	    USf(2, 2, i, j) = msing2(i) +
     +        MZ2*C2B*charge(i)*SW2 + mf(i, j)**2
	    USf(1, 2, i, j) = mf(i, j)*(Af(i, j) -
     +        dconjg(MUE)*yukfac(iiso3))

	    call ZHEEV('V', 'U', 
     +         2, USf(1, 1, i, j), 2, MSf2(1, i, j),
     +         work, lwork, rwork, svdinfo)
* transpose USf
	    fac = USf(1, 2, i, j)
	    USf(1, 2, i, j) = USf(2, 1, i, j)
	    USf(2, 1, i, j) = fac

	    if(svdinfo .ne. 0) then
	      print *,
     +          "mssm_ini: error in sfermion matrix diagonalization"
	      return 1
	    endif
	    if(MSf2(1, i, j) .lt. 0) then
	      print *, "mssm_ini: negative sfermion mass squares"
	      return 1
	    endif

	    MSf(1, i, j) = sqrt(MSf2(1, i, j))
	    MSf(2, i, j) = sqrt(MSf2(2, i, j))
	  enddo
	enddo

#ifndef NO_EXCLUSION_LIMITS
	if(MSf(1, 4, 3) .lt. MSbottom_LOWERBOUND) then
	  print *, "mssm_ini: MSf(1, 4, 3) = ", MSf(1, 4, 3),
     +      " < MSbottom_LOWERBOUND = ", MSbottom_LOWERBOUND
	  return 1
	endif

	if(MSf(1, 3, 3) .lt. MStop_LOWERBOUND) then
	  print *, "mssm_ini: MSf(1, 3, 3) = ", MSf(1, 3, 3),
     +      " < MStop_LOWERBOUND = ", MStop_LOWERBOUND
	  return 1
	endif

	if(MSf(1, 2, 1) .lt. MSlepton_LOWERBOUND) then
	  print *, "mssm_ini: MSf(1, 2, 1) = ", MSf(1, 2, 1),
     +      " < MSlepton_LOWERBOUND = ", MSlepton_LOWERBOUND
	  return 1
	endif

	if(MSf(1, 3, 1) .lt. MSquark_LOWERBOUND) then
	  print *, "mssm_ini: MSf(1, 3, 1) = ", MSf(1, 3, 1),
     +      " < MSquark_LOWERBOUND = ", MSquark_LOWERBOUND
	  return 1
	endif
#endif

	do j = 1, 3
	  MSf2(1, 1, j) = MSL2 + .5D0*MZ2*C2B
	  MSf(1, 1, j) = sqrt(MSf2(1, 1, j))
	  MSf2(2, 1, j) = -1
	  MSf(2, 1, j) = -1
	enddo

*-----------------------------------------------------------------------
* Chargino masses:

	X(1, 1) = M_2
	X(1, 2) = sqrt(2D0)*MW*SB
	X(2, 1) = sqrt(2D0)*MW*CB
	X(2, 2) = MUE
	call ZGESVD('A', 'A', 2, 2, X, 2, MCha, UCha, 2, VCha, 2,
     +    work, lwork, rwork, svdinfo)
* transpose UCha
	fac = UCha(1, 2)
	UCha(1, 2) = UCha(2, 1)
	UCha(2, 1) = fac

	if(svdinfo .ne. 0) then
	  print *, "mssm_ini: error in chargino matrix SVD"
	  return 1
	endif
	if(MCha(2) .lt. 0) then
	  print *, "mssm_ini: negative chargino mass squares"
	  return 1
	endif

#ifndef NO_EXCLUSION_LIMITS
	if(MCha(2) .lt. MChargino_LOWERBOUND) then
	  print *, "mssm_ini: MCha(2) = ", MCha(2),
     +      " < MChargino_LOWERBOUND = ", MChargino_LOWERBOUND
	  return 1
	endif
#endif

	MCha2(1) = MCha(1)**2
	MCha2(2) = MCha(2)**2

*-----------------------------------------------------------------------
* Neutralino masses:

* Neutralino mass matrix Y from Haber-Kane:
	Y(1, 1) = M_1
	Y(2, 1) = 0
	Y(3, 1) = -MZ*SW*CB
	Y(4, 1) = MZ*SW*SB
	Y(1, 2) = Y(2, 1)
	Y(2, 2) = M_2
	Y(3, 2) = MZ*CW*CB
	Y(4, 2) = -MZ*CW*SB
	Y(1, 3) = Y(3, 1)
	Y(2, 3) = Y(3, 2)
	Y(3, 3) = 0
	Y(4, 3) = -MUE
	Y(1, 4) = Y(4, 1)
	Y(2, 4) = Y(4, 2)
	Y(3, 4) = Y(4, 3)
	Y(4, 4) = 0

	do i = 1, 4
	  do j = 1, 4
	    Ysave(i, j) = Y(i, j)
	  enddo
	enddo

* Diagonalize the matrix:
* This routine calculates USVD, ZNeu, MNeu.  SVDinfo .ne. 0 => error.
* It guarantees: Y = USVD diag(MNeu) VHSVD

	call ZGESVD('A', 'A', 4, 4, Y, 4, MNeu, USVD, 4, VHSVD, 4,
     +    work, lwork, rwork, svdinfo)

	if(svdinfo .ne. 0) then
	  print *, "mssm_ini: error in neutralino matrix SVD"
	  return 1
	endif
	if(MNeu(4) .lt. 0) then
	  print *, "mssm_ini: negative neutralino mass squares"
	  return 1
	endif

#ifndef NO_EXCLUSION_LIMITS
	if(MNeu(4) .lt. MNeutralino_LOWERBOUND) then
	  print *, "mssm_ini: MNeu(4) = ", MNeu(4),
     +      " < MNeutralino_LOWERBOUND = ", MNeutralino_LOWERBOUND
	  return 1
	endif
#endif

* ZNeu_{ij} = diag(sqrt(V . U^*)^*) . V
*           = sqrt(sum_k V_{ik} U^*_{ki})^* V_{lj}
	do i = 1, 4
	  MNeu2(i) = MNeu(i)**2
	  fac = 0
	  do k = 1, 4
	    fac = fac + VHSVD(i, k)*dconjg(USVD(k, i))
	  enddo
	  fac = dconjg(sqrt(fac))
	  do j = 1, 4
	    ZNeu(i, j) = fac*VHSVD(i, j)
	  enddo
	enddo

* Now check that this is the right diagonalization matrix
* a) ZNeu^* . Y . ZNeu^dagger ?= diag(MNeu)
* b) ZNeu^dagger . ZNeu ?= 1 (unitarity)

	do i = 1, 4
	  do j = 1, 4
	    fac = 0
	    unitary = 0
	    do k = 1, 4
	      do l = 1, 4
	        fac = fac + ZNeuC(i, k)*Ysave(k, l)*ZNeuC(j, l)
	      enddo
	      unitary = unitary + ZNeuC(k, i)*ZNeu(k, j)
	    enddo
	    fac = fac/MNeu(i)
	    if(i .eq. j) then
	      fac = fac - 1
	      unitary = unitary - 1
	    endif
	    if(abs(fac) .gt. TOLERANCE) then
	      print *, "mssm_ini: ZNeu fails to diagonalize ",
     +          "the neutralino mass matrix"
	      return 1
	    endif
	    if(abs(unitary) .gt. TOLERANCE) then
	      print *, "mssm_ini: ZNeu is not unitary"
	      return 1
	    endif
	  enddo
	enddo

	end

